﻿using System;
using Ninject;
using Ninject.Activation.Blocks;
using NinjectApp.Weapon;
using NUnit.Framework;

namespace NinjectApp.Test
{
    public class ActivationBlockContext
    {
        protected StandardKernel kernel;
        protected ActivationBlock block;

        public ActivationBlockContext()
        {
            this.SetUp();
        }

        public void SetUp()
        {
            this.kernel = new StandardKernel();
            this.block = new ActivationBlock(kernel);
        }
    }

    [TestFixture]
    public class WhenBlockIsCreated : ActivationBlockContext
    {
        [Test]
        public void FirstActivatedInstanceIsReusedWithinBlock()
        {
            kernel.Bind<IWeapon>().To<Sword>();

            var weapon1 = block.Get<IWeapon>();
            var weapon2 = block.Get<IWeapon>();

            Assert.AreSame(weapon1, weapon2);
        }

        [Test]
        public void BlockDoesNotInterfereWithExternalResolution()
        {
            kernel.Bind<IWeapon>().To<Sword>();

            var weapon1 = block.Get<IWeapon>();
            var weapon2 = kernel.Get<IWeapon>();

            Assert.AreSame(weapon1, weapon2);
        }

        [Test]
        public void InstancesAreNotGarbageCollectedAsLongAsBlockRemainsAlive()
        {
            kernel.Bind<NotifiesWhenDisposed>().ToSelf();

            var instance = block.Get<NotifiesWhenDisposed>();

            GC.Collect();
            GC.WaitForPendingFinalizers();

            Assert.IsTrue(instance.IsDisposed);
        }
    }

    [TestFixture]
    public class WhenBlockIsDisposed : ActivationBlockContext
    {
        [Test]
        public void InstancesActivatedWithinBlockAreDeactivated()
        {
            kernel.Bind<NotifiesWhenDisposed>().ToSelf();

            var instance = block.Get<NotifiesWhenDisposed>();
            block.Dispose();

            Assert.IsTrue(instance.IsDisposed);
        }
    }
}
